package com.blazinggames.urcl.ui.sprite;

// Java imports
import java.awt.*;

/**
 * Ultimate Retro Common Library
 * Copyright (C) 2001, 2003-2004 Blazing Games Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ------------------------------------------------
 * 
 * A sprite that displays a block of text. The width and height of the sprite are
 * based on the text and font for the sprite. Unfortunatly, due to the way
 * java handles font information, all changes to the text and to the font will
 * require that a component class be passed.
 * <P>
 * To enhance the text, I am supporting 3 rendering formats as follows:
 * <UL>
 * <LI><B>FORMAT_PLAIN</B>Standard text</LI>
 * <LI><B>FORMAT_SHADOW</B>Text has shadow drawn in highlight color</LI>
 * <LI><B>FORMAT_OUTLINE</B>Text has shadow drawn in highlight color</LI>
 * </UL>
 *
 * @author Billy D. Spelchan
 * @version OSR1.00
 */
public class TextSprite
extends Sprite
{
// *********************
// ***** CONSTANTS *****
// *********************

    public static final Font NORMAL_FONT = new Font("Serif", 0, 16);

    public static final int FORMAT_PLAIN = 0;
    public static final int FORMAT_SHADOW = 1;
    public static final int FORMAT_OUTLINE = 2;


// *********************
// ***** VARIABLES *****
// *********************

    protected String _text;
    protected Font _font;
    protected Color _fontColor;
    protected Color _highlightColor;
    protected int _format;


// ***********************
// ***** CONSTRUCTOR *****
// ***********************

    /**
     * Default constructor creating an empty text sprite
    */
    public TextSprite()
    {
        super();
    	_text = "";
    	_font = NORMAL_FONT;
    	_fontColor = Color.yellow;
    	_highlightColor = Color.black;
    	_format = 0;
    }
    
    /**
     * Creates text sprite using provided text at provided location
     *
     * @param c required for awt calls
     * @param s text for sprite
     * @param x x location of sprite
     * @param y y location of sprite
    */
    public TextSprite(Component c, String s, int x, int y)
    {
        this();
        setText(c, s);
        move(x, y);
    }


// *********************************
// ***** METHODS - From Sprite *****
// *********************************
    /**
     * TextSprites calculate their size based on what font is used so 
     * this method is ignored.
     *
     * @param w width to scale to (should be positive)
     * @param h height to scale to (should be positive)
    */
    public void scale(int w, int h)
    {
        // DOES NOTHING - HERE DUE TO OVERLOADING REQUIREMENTS
    }

    /**
     * Sets the size and placement of the sprite. 
     * TextSprites calculate their size based on what font is used so 
     * the width and height parameters are ignored.
     *
     * @param x New x location of the sprite
     * @param y New y location of the sprite
     * @param w new width of the sprite (not used)
     * @param h new height of the sprite (not used)
    */
    public void setBounds(int x, int y, int w, int h)
    {
        move(x,y);
    }

    /**
     * Sets the size and placement of the sprite. 
     * TextSprites calculate their size based on what font is used so 
     * the width and height parameters are ignored.
     *
     * @param r New bounds for the sprite
    */
    public void setBounds(Rectangle r)
    {
        move(r.x, r.y);
    }
    
    /**
     * Draws the sprite on the provided graphics context.
     * Graphics context font and color may be changed!
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
    */
    public void render(Graphics g, Component c)
    {
        render(g, c, Sprite.NO_OFFSET);
    }
    
    /**
     * Same as render, but adjusts the sprites coordinates by adding offset to coordinates.
     *
     * @param g the graphics context to draw to
     * @param c component required by some awt draw methods
     * @param offset point relative to top left that sprite is to be drawn
    */
    public void render(Graphics g, Component c, Point offset)
    {
        if (_isVisible == false)
            return;
        super.render(g, c, offset);
        g.setFont(_font);
        FontMetrics fm = g.getFontMetrics();
        int baseline = _bounds.y + offset.y + fm.getMaxAscent();
        
        switch (_format)
        {
            case FORMAT_SHADOW:
                g.setColor(_highlightColor);
                g.drawString(_text, _bounds.x + offset.x + 1, baseline + 1);
                g.setColor(_fontColor);
                g.drawString(_text, _bounds.x + offset.x, baseline);
                break;
                
            case FORMAT_OUTLINE:
                g.setColor(_highlightColor);
                for (int cntrX = 0; cntrX < 3; ++cntrX)
                    for (int cntrY = 0; cntrY < 3; ++cntrY)
                        g.drawString(_text, _bounds.x + offset.x + cntrX, baseline + cntrY);
                g.setColor(_fontColor);
                g.drawString(_text, _bounds.x + offset.x + 1, baseline + 1);
                break;

            default:
                g.setColor(_fontColor);
                g.drawString(_text, _bounds.x + offset.x, baseline);
                break;
        }
    }


// *************************************************************
// ***** METHODS - Additional Information and modification *****
// *************************************************************

    /**
     * used internally to calculate the size of the bounds.
     * This is called by setFont and setText methods.
     *
     * @param c component required for awt calls.
    */
    protected void calculateBounds(Component c)
    {
        FontMetrics fm = c.getFontMetrics(_font);
        _bounds.setSize(fm.stringWidth(_text) + _format, 
                    fm.getMaxAscent() + fm.getMaxDescent() + _format);
    }
    
    /**
     * Sets the font that the sprite uses.
     *
     * @param c component required for awt calls
     * @param f new font to be used
    */
    public void setFont(Component c, Font f)
    {
        _font = f;
        calculateBounds(c);
        invalidate();
    }
    
    /**
     * Sets the text to be displayed
     *
     * @param c component required for awt calls
     * @param f new font to be used
    */
    public void setText(Component c, String s)
    {
        _text = s;
        calculateBounds(c);
        invalidate();
    }
    
    /** 
     * Sets the color the text is going to be drawn in
     *
     * @param c color of text
    */
    public void setFontColor(Color c)
    {
        _fontColor = c;
        invalidate();
    }
    
    /** 
     * Sets the color the text's shadow or outline is going to be drawn in.
     * A Shadow or Outline is drawn only if the appropriate format is set.
     *
     * @param c color of highlight
     * @see setFormat(int)
    */
    public void setHighlightColor(Color c)
    {
        _highlightColor = c;
        invalidate();
    }
    
    /** 
     * Sets the format based on the following constants:
     * <UL>
     * <LI><B>FORMAT_PLAIN</B>Standard text</LI>
     * <LI><B>FORMAT_SHADOW</B>Text has shadow drawn in highlight color</LI>
     * <LI><B>FORMAT_OUTLINE</B>Text has shadow drawn in highlight color</LI>
     * </UL>
     *
     * @param n format to be selected
    */
    public void setFormat(int n)
    {
        if ((n < 0) || (n > 2))
            _format = 0;
        else
            _format = n;
        invalidate();
    }
}
